@@ -216,52 +216,64 @@ def interactive(idx=0, DongleClass=RfCat, intro='', safemode=False):
     lcls = locals()
     interact(lcls, gbls)
 
-def interact(lcls, gbls):
-    shellexception = None
+STYPE_NONE = 0
+STYPE_IPYTHON = 1
+STYPE_IPYTHON811P = 2
+STYPE_CODE_INTERACT = 3
 
+def interact(lcls, gbls, intro=""):
+    shelltype = STYPE_NONE
     try:
-        import IPython
-        if IPython.core.getipython.get_ipython() is None:
-            ipsh = IPython.terminal.embed.InteractiveShellEmbed.instance()
-        else:
-            ipsh = IPython.terminal.embed.InteractiveShellEmbed()
-       
-        ipsh.user_global_ns.update(gbls)    
-        ipsh.user_global_ns.update(lcls)
-        ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
-        ipsh.mainloop()
-    except ImportError as e:
-        shellexception = e
+        from IPython.terminal.embed import embed
+        print(intro)
+        shelltype = STYPE_IPYTHON811P
 
-    if shellexception:
+    except ImportError as e:
         try:
             import IPython.Shell
             ipsh = IPython.Shell.IPShell(argv=[''], user_ns=lcls, user_global_ns=gbls)
-            ipsh.mainloop()
+            print(intro)
+            shelltype = STYPE_IPYTHON
 
         except ImportError as e:
-            shellexception = e
+            try:
+                from IPython.terminal.interactiveshell import TerminalInteractiveShell
+                ipsh = TerminalInteractiveShell()
+                ipsh.user_global_ns.update(gbls)
+                ipsh.user_global_ns.update(lcls)
+                ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
+                shelltype = STYPE_IPYTHON
+                print(intro)
 
-    if shellexception:
-        try:
-            import IPython
-            if IPython.core.getipython.get_ipython() is None:
-                ipsh = IPython.terminal.embed.InteractiveShellEmbed.instance()
-            else:
-                ipsh = IPython.terminal.embed.InteractiveShellEmbed()
-            ipsh.user_global_ns.update(gbls)
-            ipsh.user_global_ns.update(lcls)
-            ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
+            except ImportError as e:
+                try:
+                    from IPython.frontend.terminal.interactiveshell import TerminalInteractiveShell
+                    ipsh = TerminalInteractiveShell()
+                    ipsh.user_global_ns.update(gbls)
+                    ipsh.user_global_ns.update(lcls)
+                    ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
+                    shelltype = STYPE_IPYTHON
 
-            ipsh.mainloop()
-        except ImportError as e:
-            shellexception = e
+                    print(intro)
+                except ImportError as e:
+                    print(e)
+                    shell = code.InteractiveConsole(gbls)
+                    shelltype = STYPE_IPYTHON
+                    print(intro)
 
-    if shellexception:
+    if shelltype == STYPE_IPYTHON811P:
+        embed()
+
+    elif shelltype == STYPE_IPYTHON:
+        ipsh.mainloop()
+
+    elif shelltype == STYPE_CODE_INTERACT:
         print("falling back to straight Python... (%r)" % shellexception)
-        shell = code.InteractiveConsole(gbls)
         shell.interact()
 
+    else:
+        print("SORRY, NO INTERACTIVE OPTIONS AVAILABLE!!  wtfo?")
+
 
 if __name__ == "__main__":
     idx = 0
